@@ -1,14 +1,16 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
-#include "zeek-config.h"
 #include "zeek/CCL.h"
 
+#include "zeek/zeek-config.h"
+
 #include <algorithm>
 
-#include "zeek/RE.h"
 #include "zeek/DFA.h"
+#include "zeek/RE.h"
 
-namespace zeek::detail {
+namespace zeek::detail
+	{
 
 CCL::CCL()
 	{
@@ -34,8 +36,8 @@ void CCL::Add(int sym)
 	auto sym_p = static_cast<std::intptr_t>(sym);
 
 	// Check to see if the character is already in the ccl.
-	for ( auto sym : *syms )
-		if ( sym == sym_p )
+	for ( auto sym_entry : *syms )
+		if ( sym_entry == sym_p )
 			return;
 
 	syms->push_back(sym_p);
@@ -48,7 +50,8 @@ void CCL::Sort()
 
 unsigned int CCL::MemoryAllocation() const
 	{
-	return padded_sizeof(*this) + padded_sizeof(*syms) + util::pad_size(syms->size() * sizeof(int_list::value_type));
+	return padded_sizeof(*this) + padded_sizeof(*syms) +
+	       util::pad_size(syms->size() * sizeof(int_list::value_type));
 	}
 
-} // namespace zeek::detail
+	} // namespace zeek::detail
