@@ -1,30 +1,40 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
-#include "Flare.h"
-#include "Reporter.h"
-#include <unistd.h>
-#include <fcntl.h>
+#include "zeek/Flare.h"
+
 #include <errno.h>
+#include <fcntl.h>
+#include <unistd.h>
 
-using namespace bro;
+#include "zeek/Reporter.h"
 
-Flare::Flare()
-	: pipe(FD_CLOEXEC, FD_CLOEXEC, O_NONBLOCK, O_NONBLOCK)
+namespace zeek::detail
 	{
-	}
 
-static void bad_pipe_op(const char* which)
+Flare::Flare() : pipe(FD_CLOEXEC, FD_CLOEXEC, O_NONBLOCK, O_NONBLOCK) { }
+
+[[noreturn]] static void bad_pipe_op(const char* which, bool signal_safe)
 	{
+	if ( signal_safe )
+		abort();
+
 	char buf[256];
-	bro_strerror_r(errno, buf, sizeof(buf));
-	reporter->FatalErrorWithCore("unexpected pipe %s failure: %s", which, buf);
+	util::zeek_strerror_r(errno, buf, sizeof(buf));
+
+	if ( reporter )
+		reporter->FatalErrorWithCore("unexpected pipe %s failure: %s", which, buf);
+	else
+		{
+		fprintf(stderr, "unexpected pipe %s failure: %s", which, buf);
+		abort();
+		}
 	}
 
-void Flare::Fire()
+void Flare::Fire(bool signal_safe)
 	{
 	char tmp = 0;
 
-	for ( ; ; )
+	for ( ;; )
 		{
 		int n = write(pipe.WriteFD(), &tmp, 1);
 
@@ -42,24 +52,28 @@ void Flare::Fire()
 				// Interrupted: try again.
 				continue;
 
-			bad_pipe_op("write");
+			bad_pipe_op("write", signal_safe);
 			}
 
 		// No error, but didn't write a byte: try again.
 		}
 	}
 
-void Flare::Extinguish()
+int Flare::Extinguish(bool signal_safe)
 	{
+	int rval = 0;
 	char tmp[256];
 
-	for ( ; ; )
+	for ( ;; )
 		{
 		int n = read(pipe.ReadFD(), &tmp, sizeof(tmp));
 
 		if ( n >= 0 )
+			{
+			rval += n;
 			// Pipe may not be empty yet: try again.
 			continue;
+			}
 
 		if ( errno == EAGAIN )
 			// Success: pipe is now empty.
@@ -69,6 +83,10 @@ void Flare::Extinguish()
 			// Interrupted: try again.
 			continue;
 
-		bad_pipe_op("read");
+		bad_pipe_op("read", signal_safe);
 		}
+
+	return rval;
 	}
+
+	} // namespace zeek::detail
