@@ -1,17 +1,17 @@
-#include "zeek-config.h"
+#include "zeek/analyzer/protocol/mime/MIME.h"
 
-#include "NetVar.h"
-#include "MIME.h"
-#include "Event.h"
-#include "Reporter.h"
-#include "digest.h"
-#include "file_analysis/Manager.h"
+#include "zeek/zeek-config.h"
 
-#include "events.bif.h"
+#include "zeek/Base64.h"
+#include "zeek/NetVar.h"
+#include "zeek/Reporter.h"
+#include "zeek/analyzer/protocol/mime/events.bif.h"
+#include "zeek/digest.h"
+#include "zeek/file_analysis/Manager.h"
 
 // Here are a few things to do:
 //
-// 1. Add a Bro internal function 'stop_deliver_data_of_entity' so
+// 1. Add a Zeek internal function 'stop_deliver_data_of_entity' so
 // that the engine does not decode and deliver further data for the
 // entity (which may speed up the engine by avoiding copying).
 //
@@ -19,115 +19,115 @@
 // headers of form: <name>=<value>; <param_1>=<param_val_1>;
 // <param_2>=<param_val_2>; ... (so that
 
-namespace analyzer { namespace mime {
+namespace zeek::analyzer::mime
+	{
 
-static const data_chunk_t null_data_chunk = { 0, 0 };
+static const data_chunk_t null_data_chunk = {0, nullptr};
 
 int mime_header_only = 0;
 int mime_decode_data = 1;
 int mime_submit_data = 1;
 
-enum MIME_HEADER_FIELDS {
+enum MIME_HEADER_FIELDS
+	{
 	MIME_CONTENT_TYPE,
 	MIME_CONTENT_TRANSFER_ENCODING,
 	MIME_FIELD_OTHER,
-};
+	};
 
-enum MIME_CONTENT_SUBTYPE {
-	CONTENT_SUBTYPE_MIXED,		// for multipart
-	CONTENT_SUBTYPE_ALTERNATIVE,	// for multipart
-	CONTENT_SUBTYPE_DIGEST,		// for multipart
+enum MIME_CONTENT_SUBTYPE
+	{
+	CONTENT_SUBTYPE_MIXED, // for multipart
+	CONTENT_SUBTYPE_ALTERNATIVE, // for multipart
+	CONTENT_SUBTYPE_DIGEST, // for multipart
 
-	CONTENT_SUBTYPE_RFC822,		// for message
-	CONTENT_SUBTYPE_PARTIAL,	// for message
-	CONTENT_SUBTYPE_EXTERNAL_BODY,	// for message
+	CONTENT_SUBTYPE_RFC822, // for message
+	CONTENT_SUBTYPE_PARTIAL, // for message
+	CONTENT_SUBTYPE_EXTERNAL_BODY, // for message
 
-	CONTENT_SUBTYPE_PLAIN,		// for text
+	CONTENT_SUBTYPE_PLAIN, // for text
 
 	CONTENT_SUBTYPE_OTHER,
-};
+	};
 
-enum MIME_CONTENT_ENCODING {
+enum MIME_CONTENT_ENCODING
+	{
 	CONTENT_ENCODING_7BIT,
 	CONTENT_ENCODING_8BIT,
 	CONTENT_ENCODING_BINARY,
 	CONTENT_ENCODING_QUOTED_PRINTABLE,
 	CONTENT_ENCODING_BASE64,
 	CONTENT_ENCODING_OTHER,
-};
+	};
 
-enum MIME_BOUNDARY_DELIMITER {
+enum MIME_BOUNDARY_DELIMITER
+	{
 	NOT_MULTIPART_BOUNDARY,
 	MULTIPART_BOUNDARY,
 	MULTIPART_CLOSING_BOUNDARY,
-};
+	};
 
 static const char* MIMEHeaderName[] = {
 	"content-type",
 	"content-transfer-encoding",
-	0,
+	nullptr,
 };
 
 static const char* MIMEContentTypeName[] = {
 	"MULTIPART",
 	"MESSAGE",
 	"TEXT",
-	0,
+	nullptr,
 };
 
 static const char* MIMEContentSubtypeName[] = {
-	"MIXED",		// for multipart
-	"ALTERNATIVE",		// for multipart
-	"DIGEST",		// for multipart
+	"MIXED", // for multipart
+	"ALTERNATIVE", // for multipart
+	"DIGEST", // for multipart
 
-	"RFC822",		// for message
-	"PARTIAL",		// for message
-	"EXTERNAL-BODY",	// for message
+	"RFC822", // for message
+	"PARTIAL", // for message
+	"EXTERNAL-BODY", // for message
 
-	"PLAIN",		// for text
+	"PLAIN", // for text
 
-	0,			// other
+	nullptr, // other
 };
 
 static const char* MIMEContentEncodingName[] = {
-	"7BIT",
-	"8BIT",
-	"BINARY",
-	"QUOTED-PRINTABLE",
-	"BASE64",
-	0,
+	"7BIT", "8BIT", "BINARY", "QUOTED-PRINTABLE", "BASE64", nullptr,
 };
 
-int is_null_data_chunk(data_chunk_t b)
+bool is_null_data_chunk(data_chunk_t b)
 	{
-	return b.data == 0;
+	return b.data == nullptr;
 	}
 
-int is_lws(char ch)
+bool is_lws(char ch)
 	{
 	return ch == 9 || ch == 32;
 	}
 
-StringVal* new_string_val(int length, const char* data)
+StringValPtr to_string_val(int length, const char* data)
 	{
-	return new StringVal(length, data);
+	return make_intrusive<StringVal>(length, data);
 	}
 
-StringVal* new_string_val(const char* data, const char* end_of_data)
+StringValPtr to_string_val(const char* data, const char* end_of_data)
 	{
-	return new StringVal(end_of_data - data, data);
+	return make_intrusive<StringVal>(end_of_data - data, data);
 	}
 
-StringVal* new_string_val(const data_chunk_t buf)
+StringValPtr to_string_val(const data_chunk_t buf)
 	{
-	return new_string_val(buf.length, buf.data);
+	return to_string_val(buf.length, buf.data);
 	}
 
-static data_chunk_t get_data_chunk(BroString* s)
+static data_chunk_t get_data_chunk(String* s)
 	{
 	data_chunk_t b;
 	b.length = s->Len();
-	b.data = (const char*) s->Bytes();
+	b.data = (const char*)s->Bytes();
 	return b;
 	}
 
@@ -141,8 +141,7 @@ int fputs(data_chunk_t b, FILE* fp)
 
 void MIME_Mail::Undelivered(int len)
 	{
-	cur_entity_id = file_mgr->Gap(cur_entity_len, len,
-	                              analyzer->GetAnalyzerTag(), analyzer->Conn(),
+	cur_entity_id = file_mgr->Gap(cur_entity_len, len, analyzer->GetAnalyzerTag(), analyzer->Conn(),
 	                              is_orig, cur_entity_id);
 	}
 
@@ -183,21 +182,22 @@ int MIME_skip_comments(int len, const char* data)
 	int par = 0;
 	for ( int i = 0; i < len; ++i )
 		{
-		switch ( data[i] ) {
-		case '(':
-			++par;
-			break;
+		switch ( data[i] )
+			{
+			case '(':
+				++par;
+				break;
 
-		case ')':
-			--par;
-			if ( par == 0 )
-				return i + 1;
-			break;
+			case ')':
+				--par;
+				if ( par == 0 )
+					return i + 1;
+				break;
 
-		case '\\':
-			++i;
-			break;
-		}
+			case '\\':
+				++i;
+				break;
+			}
 		}
 
 	return len;
@@ -250,30 +250,29 @@ int MIME_get_field_name(int len, const char* data, data_chunk_t* name)
 	}
 
 // See RFC 2045, page 12.
-int MIME_is_tspecial (char ch, bool is_boundary = false)
+static bool MIME_is_tspecial(char ch, bool is_boundary = false)
 	{
 	if ( is_boundary )
 		return ch == '"';
 	else
-		return ch == '(' || ch == ')' || ch == '<' || ch == '>' || ch == '@' ||
-		       ch == ',' || ch == ';' || ch == ':' || ch == '\\' || ch == '"' ||
-		       ch == '/' || ch == '[' || ch == ']' || ch == '?' || ch == '=';
+		return ch == '(' || ch == ')' || ch == '<' || ch == '>' || ch == '@' || ch == ',' ||
+		       ch == ';' || ch == ':' || ch == '\\' || ch == '"' || ch == '/' || ch == '[' ||
+		       ch == ']' || ch == '?' || ch == '=';
 	}
 
-int MIME_is_field_name_char (char ch)
+bool MIME_is_field_name_char(char ch)
 	{
 	return ch >= 33 && ch <= 126 && ch != ':';
 	}
 
-int MIME_is_token_char (char ch, bool is_boundary = false)
+static bool MIME_is_token_char(char ch, bool is_boundary = false)
 	{
 	return ch >= 33 && ch <= 126 && ! MIME_is_tspecial(ch, is_boundary);
 	}
 
 // See RFC 2045, page 12.
 // A token is composed of characters that are not SPACE, CTLs or tspecials
-int MIME_get_token(int len, const char* data, data_chunk_t* token,
-                   bool is_boundary)
+int MIME_get_token(int len, const char* data, data_chunk_t* token, bool is_boundary)
 	{
 	int i = 0;
 
@@ -356,26 +355,27 @@ int MIME_get_quoted_string(int len, const char* data, data_chunk_t* str)
 
 	for ( int i = 1; i < len; ++i )
 		{
-		switch ( data[i] ) {
-		case '"':
-			str->data = data + 1;
-			str->length = i - 1;
-			return offset + i + 1;
+		switch ( data[i] )
+			{
+			case '"':
+				str->data = data + 1;
+				str->length = i - 1;
+				return offset + i + 1;
 
-		case '\\':
-			++i;
-			break;
-		}
+			case '\\':
+				++i;
+				break;
+			}
 		}
 
 	return -1;
 	}
 
-int MIME_get_value(int len, const char* data, BroString*& buf, bool is_boundary)
+int MIME_get_value(int len, const char* data, String*& buf, bool is_boundary)
 	{
 	int offset = 0;
 
-	if ( ! is_boundary )	// For boundaries, simply accept everything.
+	if ( ! is_boundary ) // For boundaries, simply accept everything.
 		offset = MIME_skip_lws_comments(len, data);
 
 	len -= offset;
@@ -399,7 +399,7 @@ int MIME_get_value(int len, const char* data, BroString*& buf, bool is_boundary)
 		if ( end < 0 )
 			return -1;
 
-		buf = new BroString((const u_char*)str.data, str.length, 1);
+		buf = new String((const u_char*)str.data, str.length, true);
 		return offset + end;
 		}
 	}
@@ -407,10 +407,10 @@ int MIME_get_value(int len, const char* data, BroString*& buf, bool is_boundary)
 // Decode each quoted-pair: a '\' followed by a character by the
 // quoted character. The decoded string is returned.
 
-BroString* MIME_decode_quoted_pairs(data_chunk_t buf)
+String* MIME_decode_quoted_pairs(data_chunk_t buf)
 	{
 	const char* data = buf.data;
-	char* dest = new char[buf.length+1];
+	char* dest = new char[buf.length + 1];
 	int j = 0;
 	for ( int i = 0; i < buf.length; ++i )
 		if ( data[i] == '\\' )
@@ -427,17 +427,12 @@ BroString* MIME_decode_quoted_pairs(data_chunk_t buf)
 			dest[j++] = data[i];
 	dest[j] = 0;
 
-	return new BroString(1, (byte_vec) dest, j);
+	return new String(true, (byte_vec)dest, j);
 	}
 
-
-} } // namespace analyzer::*
-
-using namespace analyzer::mime;
-
 MIME_Multiline::MIME_Multiline()
 	{
-	line = 0;
+	line = nullptr;
 	}
 
 MIME_Multiline::~MIME_Multiline()
@@ -448,13 +443,13 @@ MIME_Multiline::~MIME_Multiline()
 
 void MIME_Multiline::append(int len, const char* data)
 	{
-	buffer.push_back(new BroString((const u_char*) data, len, 1));
+	buffer.push_back(new String((const u_char*)data, len, true));
 	}
 
-BroString* MIME_Multiline::get_concatenated_line()
+String* MIME_Multiline::get_concatenated_line()
 	{
-	if ( buffer.size() == 0 )
-		return 0;
+	if ( buffer.empty() )
+		return nullptr;
 
 	delete line;
 	line = concatenate(buffer);
@@ -462,21 +457,21 @@ BroString* MIME_Multiline::get_concatenated_line()
 	return line;
 	}
 
-
 MIME_Header::MIME_Header(MIME_Multiline* hl)
 	{
 	lines = hl;
 	name = value = value_token = rest_value = null_data_chunk;
 
-	BroString* s = hl->get_concatenated_line();
+	String* s = hl->get_concatenated_line();
 	int len = s->Len();
-	const char* data = (const char*) s->Bytes();
+	const char* data = (const char*)s->Bytes();
 
 	int offset = MIME_get_field_name(len, data, &name);
 	if ( offset < 0 )
 		return;
 
-	len -= offset; data += offset;
+	len -= offset;
+	data += offset;
 	offset = MIME_skip_lws_comments(len, data);
 
 	if ( offset < len && data[offset] == ':' )
@@ -537,6 +532,8 @@ MIME_Entity::MIME_Entity(MIME_Message* output_message, MIME_Entity* parent_entit
 	message = output_message;
 	if ( parent )
 		content_encoding = parent->ContentTransferEncoding();
+
+	want_all_headers = (bool)mime_all_headers;
 	}
 
 void MIME_Entity::init()
@@ -544,53 +541,52 @@ void MIME_Entity::init()
 	in_header = 1;
 	end_of_data = 0;
 
-	current_header_line = 0;
+	current_header_line = nullptr;
 	current_field_type = MIME_FIELD_OTHER;
 
 	need_to_parse_parameters = 0;
 
-	content_type_str = new StringVal("TEXT");
-	content_subtype_str = new StringVal("PLAIN");
+	content_type_str = make_intrusive<StringVal>("TEXT");
+	content_subtype_str = make_intrusive<StringVal>("PLAIN");
 
-	content_encoding_str = 0;
-	multipart_boundary = 0;
+	content_encoding_str = nullptr;
+	multipart_boundary = nullptr;
 	content_type = CONTENT_TYPE_TEXT;
 	content_subtype = CONTENT_SUBTYPE_PLAIN;
 	content_encoding = CONTENT_ENCODING_OTHER;
 
-	parent = 0;
-	current_child_entity = 0;
+	parent = nullptr;
+	current_child_entity = nullptr;
 
-	base64_decoder = 0;
+	base64_decoder = nullptr;
 
 	data_buf_length = 0;
-	data_buf_data = 0;
+	data_buf_data = nullptr;
 	data_buf_offset = -1;
 
-	message = 0;
+	message = nullptr;
 	delay_adding_implicit_CRLF = false;
+	want_all_headers = false;
 	}
 
 MIME_Entity::~MIME_Entity()
 	{
 	if ( ! end_of_data )
-		reporter->AnalyzerError(message ? message->GetAnalyzer() : 0,
-		            "missing MIME_Entity::EndOfData() before ~MIME_Entity");
+		reporter->AnalyzerError(message ? message->GetAnalyzer() : nullptr,
+		                        "missing MIME_Entity::EndOfData() before ~MIME_Entity");
 
 	delete current_header_line;
-	Unref(content_type_str);
-	Unref(content_subtype_str);
 	delete content_encoding_str;
 	delete multipart_boundary;
 
-	for ( unsigned int i = 0; i < headers.size(); ++i )
-		delete headers[i];
+	for ( auto& header : headers )
+		delete header;
 	headers.clear();
 
 	delete base64_decoder;
 	}
 
-void MIME_Entity::Deliver(int len, const char* data, int trailing_CRLF)
+void MIME_Entity::Deliver(int len, const char* data, bool trailing_CRLF)
 	{
 	if ( in_header )
 		{
@@ -644,13 +640,12 @@ void MIME_Entity::EndOfData()
 		FinishHeader();
 		in_header = 0;
 		SubmitAllHeaders();
-		message->SubmitEvent(MIME_EVENT_ILLEGAL_FORMAT,
-					"entity body missing");
+		message->SubmitEvent(MIME_EVENT_ILLEGAL_FORMAT, "entity body missing");
 		}
 
 	else
 		{
-		if ( current_child_entity != 0 )
+		if ( current_child_entity != nullptr )
 			{
 			if ( content_type == CONTENT_TYPE_MULTIPART )
 				IllegalFormat("multipart closing boundary delimiter missing");
@@ -663,37 +658,37 @@ void MIME_Entity::EndOfData()
 		FlushData();
 		}
 
-	message->EndEntity (this);
+	message->EndEntity(this);
 	}
 
-void MIME_Entity::NewDataLine(int len, const char* data, int trailing_CRLF)
+void MIME_Entity::NewDataLine(int len, const char* data, bool trailing_CRLF)
 	{
 	if ( content_type == CONTENT_TYPE_MULTIPART )
 		{
-		switch ( CheckBoundaryDelimiter(len, data) ) {
+		switch ( CheckBoundaryDelimiter(len, data) )
+			{
 			case MULTIPART_BOUNDARY:
-				if ( current_child_entity != 0 )
+				if ( current_child_entity != nullptr )
 					EndChildEntity();
 				if ( ! end_of_data )
 					BeginChildEntity();
 				return;
 
 			case MULTIPART_CLOSING_BOUNDARY:
-				if ( current_child_entity != 0 )
+				if ( current_child_entity != nullptr )
 					EndChildEntity();
 				EndOfData();
 				return;
-		}
+			}
 		}
 
-	if ( content_type == CONTENT_TYPE_MULTIPART ||
-	     content_type == CONTENT_TYPE_MESSAGE )
+	if ( content_type == CONTENT_TYPE_MULTIPART || content_type == CONTENT_TYPE_MESSAGE )
 		{
 		// Here we ignore the difference among 7bit, 8bit and
 		// binary encoding, and thus do not need to decode
 		// before passing the data to child.
 
-		if ( current_child_entity != 0 )
+		if ( current_child_entity != nullptr )
 			// Data before the first or after the last
 			// boundary delimiter are ignored
 			current_child_entity->Deliver(len, data, trailing_CRLF);
@@ -720,7 +715,7 @@ void MIME_Entity::NewHeader(int len, const char* data)
 
 void MIME_Entity::ContHeader(int len, const char* data)
 	{
-	if ( current_header_line == 0 )
+	if ( current_header_line == nullptr )
 		{
 		IllegalFormat("first header line starts with linear whitespace");
 
@@ -735,17 +730,21 @@ void MIME_Entity::ContHeader(int len, const char* data)
 
 void MIME_Entity::FinishHeader()
 	{
-	if ( current_header_line == 0 )
+	if ( current_header_line == nullptr )
 		return;
 
 	MIME_Header* h = new MIME_Header(current_header_line);
-	current_header_line = 0;
+	current_header_line = nullptr;
 
 	if ( ! is_null_data_chunk(h->get_name()) )
 		{
 		ParseMIMEHeader(h);
 		SubmitHeader(h);
-		headers.push_back(h);
+
+		if ( want_all_headers )
+			headers.push_back(h);
+		else
+			delete h;
 		}
 	else
 		delete h;
@@ -756,7 +755,7 @@ int MIME_Entity::LookupMIMEHeaderName(data_chunk_t name)
 	// A linear lookup should be fine for now.
 	// header names are case-insensitive (RFC 822, 2822, 2045).
 
-	for ( int i = 0; MIMEHeaderName[i] != 0; ++i )
+	for ( int i = 0; MIMEHeaderName[i] != nullptr; ++i )
 		if ( istrequal(name, MIMEHeaderName[i]) )
 			return i;
 	return -1;
@@ -764,12 +763,13 @@ int MIME_Entity::LookupMIMEHeaderName(data_chunk_t name)
 
 void MIME_Entity::ParseMIMEHeader(MIME_Header* h)
 	{
-	if ( h == 0 )
+	if ( h == nullptr )
 		return;
 
 	current_field_type = LookupMIMEHeaderName(h->get_name());
 
-	switch ( current_field_type ) {
+	switch ( current_field_type )
+		{
 		case MIME_CONTENT_TYPE:
 			ParseContentTypeField(h);
 			break;
@@ -777,10 +777,10 @@ void MIME_Entity::ParseMIMEHeader(MIME_Header* h)
 		case MIME_CONTENT_TRANSFER_ENCODING:
 			ParseContentEncodingField(h);
 			break;
-	}
+		}
 	}
 
-int MIME_Entity::ParseContentTypeField(MIME_Header* h)
+bool MIME_Entity::ParseContentTypeField(MIME_Header* h)
 	{
 	data_chunk_t val = h->get_value();
 	int len = val.length;
@@ -793,15 +793,15 @@ int MIME_Entity::ParseContentTypeField(MIME_Header* h)
 	if ( offset < 0 )
 		{
 		IllegalFormat("media type/subtype not found in content type");
-		return 0;
+		return false;
 		}
 	data += offset;
 	len -= offset;
 
-	Unref(content_type_str);
-	content_type_str = (new StringVal(ty.length, ty.data))->ToUpper();
-	Unref(content_subtype_str);
-	content_subtype_str = (new StringVal(subty.length, subty.data))->ToUpper();
+	content_type_str = make_intrusive<StringVal>(ty.length, ty.data);
+	content_type_str->ToUpper();
+	content_subtype_str = make_intrusive<StringVal>(subty.length, subty.data);
+	content_subtype_str->ToUpper();
 
 	ParseContentType(ty, subty);
 
@@ -811,15 +811,16 @@ int MIME_Entity::ParseContentTypeField(MIME_Header* h)
 
 	if ( content_type == CONTENT_TYPE_MULTIPART && ! multipart_boundary )
 		{
-		IllegalFormat("boundary delimiter is not specified for a multipart entity -- content is treated as type application/octet-stream");
+		IllegalFormat("boundary delimiter is not specified for a multipart entity -- content is "
+		              "treated as type application/octet-stream");
 		content_type = CONTENT_TYPE_OTHER;
 		content_subtype = CONTENT_SUBTYPE_OTHER;
 		}
 
-	return 1;
+	return true;
 	}
 
-int MIME_Entity::ParseContentEncodingField(MIME_Header* h)
+bool MIME_Entity::ParseContentEncodingField(MIME_Header* h)
 	{
 	data_chunk_t enc;
 
@@ -827,11 +828,11 @@ int MIME_Entity::ParseContentEncodingField(MIME_Header* h)
 	if ( is_null_data_chunk(enc) )
 		{
 		IllegalFormat("encoding type not found in content encoding");
-		return 0;
+		return false;
 		}
 
 	delete content_encoding_str;
-	content_encoding_str = new BroString((const u_char*)enc.data, enc.length, true);
+	content_encoding_str = new String((const u_char*)enc.data, enc.length, true);
 	ParseContentEncoding(enc);
 
 	if ( need_to_parse_parameters )
@@ -841,14 +842,14 @@ int MIME_Entity::ParseContentEncodingField(MIME_Header* h)
 			ParseFieldParameters(val.length, val.data);
 		}
 
-	return 1;
+	return true;
 	}
 
-int MIME_Entity::ParseFieldParameters(int len, const char* data)
+bool MIME_Entity::ParseFieldParameters(int len, const char* data)
 	{
 	data_chunk_t attr;
 
-	while ( 1 )
+	while ( true )
 		{
 		int offset = MIME_skip_lws_comments(len, data);
 		if ( offset < 0 || offset >= len || data[offset] != ';' )
@@ -862,7 +863,7 @@ int MIME_Entity::ParseFieldParameters(int len, const char* data)
 		if ( offset < 0 )
 			{
 			IllegalFormat("attribute name not found in parameter specification");
-			return 0;
+			return false;
 			}
 
 		data += offset;
@@ -879,10 +880,9 @@ int MIME_Entity::ParseFieldParameters(int len, const char* data)
 		data += offset;
 		len -= offset;
 
-		BroString* val = 0;
+		String* val = nullptr;
 
-		if ( current_field_type == MIME_CONTENT_TYPE &&
-		     content_type == CONTENT_TYPE_MULTIPART &&
+		if ( current_field_type == MIME_CONTENT_TYPE && content_type == CONTENT_TYPE_MULTIPART &&
 		     istrequal(attr, "boundary") )
 			{
 			// token or quoted-string (and some lenience for characters
@@ -897,8 +897,7 @@ int MIME_Entity::ParseFieldParameters(int len, const char* data)
 
 			data_chunk_t vd = get_data_chunk(val);
 			delete multipart_boundary;
-			multipart_boundary = new BroString((const u_char*)vd.data,
-			                                   vd.length, 1);
+			multipart_boundary = new String((const u_char*)vd.data, vd.length, true);
 			}
 		else
 			// token or quoted-string
@@ -916,7 +915,7 @@ int MIME_Entity::ParseFieldParameters(int len, const char* data)
 		delete val;
 		}
 
-	return 1;
+	return true;
 	}
 
 void MIME_Entity::ParseContentType(data_chunk_t type, data_chunk_t sub_type)
@@ -934,7 +933,8 @@ void MIME_Entity::ParseContentType(data_chunk_t type, data_chunk_t sub_type)
 
 	content_subtype = i;
 
-	switch ( content_type ) {
+	switch ( content_type )
+		{
 		case CONTENT_TYPE_MULTIPART:
 		case CONTENT_TYPE_MESSAGE:
 			need_to_parse_parameters = 1;
@@ -943,7 +943,7 @@ void MIME_Entity::ParseContentType(data_chunk_t type, data_chunk_t sub_type)
 		default:
 			need_to_parse_parameters = 0;
 			break;
-	}
+		}
 	}
 
 void MIME_Entity::ParseContentEncoding(data_chunk_t encoding_mechanism)
@@ -968,7 +968,8 @@ int MIME_Entity::CheckBoundaryDelimiter(int len, const char* data)
 
 	if ( len >= 2 && data[0] == '-' && data[1] == '-' )
 		{
-		len -= 2; data += 2;
+		len -= 2;
+		data += 2;
 
 		data_chunk_t delim = get_data_chunk(multipart_boundary);
 
@@ -992,16 +993,16 @@ int MIME_Entity::CheckBoundaryDelimiter(int len, const char* data)
 	return NOT_MULTIPART_BOUNDARY;
 	}
 
-
 // trailing_CRLF indicates whether an implicit CRLF sequence follows data
 // (the CRLF sequence is not included in data).
 
-void MIME_Entity::DecodeDataLine(int len, const char* data, int trailing_CRLF)
+void MIME_Entity::DecodeDataLine(int len, const char* data, bool trailing_CRLF)
 	{
 	if ( ! mime_submit_data )
 		return;
 
-	switch ( content_encoding ) {
+	switch ( content_encoding )
+		{
 		case CONTENT_ENCODING_QUOTED_PRINTABLE:
 			DecodeQuotedPrintable(len, data);
 			break;
@@ -1016,11 +1017,11 @@ void MIME_Entity::DecodeDataLine(int len, const char* data, int trailing_CRLF)
 		case CONTENT_ENCODING_OTHER:
 			DecodeBinary(len, data, trailing_CRLF);
 			break;
-	}
+		}
 	FlushData();
 	}
 
-void MIME_Entity::DecodeBinary(int len, const char* data, int trailing_CRLF)
+void MIME_Entity::DecodeBinary(int len, const char* data, bool trailing_CRLF)
 	{
 	if ( delay_adding_implicit_CRLF )
 		{
@@ -1033,8 +1034,7 @@ void MIME_Entity::DecodeBinary(int len, const char* data, int trailing_CRLF)
 
 	if ( trailing_CRLF )
 		{
-		if ( Parent() &&
-		     Parent()->MIMEContentType() == mime::CONTENT_TYPE_MULTIPART )
+		if ( Parent() && Parent()->MIMEContentType() == mime::CONTENT_TYPE_MULTIPART )
 			{
 			// For multipart body content, we want to keep all implicit CRLFs
 			// except for the last because that one belongs to the multipart
@@ -1075,8 +1075,8 @@ void MIME_Entity::DecodeQuotedPrintable(int len, const char* data)
 				if ( i + 2 < len )
 					{
 					int a, b;
-					a = decode_hex(data[i+1]);
-					b = decode_hex(data[i+2]);
+					a = util::decode_hex(data[i + 1]);
+					b = util::decode_hex(data[i + 2]);
 
 					if ( a >= 0 && b >= 0 )
 						{
@@ -1090,15 +1090,16 @@ void MIME_Entity::DecodeQuotedPrintable(int len, const char* data)
 					{
 					// Follows suggestions for a robust
 					// decoder. See RFC 2045 page 22.
-					IllegalEncoding("= is not followed by two hexadecimal digits in quoted-printable encoding");
+					IllegalEncoding(
+						"= is not followed by two hexadecimal digits in quoted-printable encoding");
 					DataOctet(data[i]);
 					}
 				}
 			}
 
 		else if ( (data[i] >= 33 && data[i] <= 60) ||
-			   // except controls, whitespace and '='
-			  (data[i] >= 62 && data[i] <= 126) )
+		          // except controls, whitespace and '='
+		          (data[i] >= 62 && data[i] <= 126) )
 			DataOctet(data[i]);
 
 		else if ( data[i] == HT || data[i] == SP )
@@ -1106,7 +1107,8 @@ void MIME_Entity::DecodeQuotedPrintable(int len, const char* data)
 
 		else
 			{
-			IllegalEncoding(fmt("control characters in quoted-printable encoding: %d", (int) (data[i])));
+			IllegalEncoding(
+				util::fmt("control characters in quoted-printable encoding: %d", (int)(data[i])));
 			DataOctet(data[i]);
 			}
 		}
@@ -1129,7 +1131,8 @@ void MIME_Entity::DecodeBase64(int len, const char* data)
 		char* prbuf = rbuf;
 		int decoded = base64_decoder->Decode(len, data, &rlen, &prbuf);
 		DataOctets(rlen, rbuf);
-		len -= decoded; data += decoded;
+		len -= decoded;
+		data += decoded;
 		}
 	}
 
@@ -1149,7 +1152,7 @@ void MIME_Entity::StartDecodeBase64()
 		return;
 		}
 
-	base64_decoder = new Base64Converter(analyzer->Conn());
+	base64_decoder = new zeek::detail::Base64Converter(analyzer->Conn());
 	}
 
 void MIME_Entity::FinishDecodeBase64()
@@ -1168,20 +1171,20 @@ void MIME_Entity::FinishDecodeBase64()
 		}
 
 	delete base64_decoder;
-	base64_decoder = 0;
+	base64_decoder = nullptr;
 	}
 
-int MIME_Entity::GetDataBuffer()
+bool MIME_Entity::GetDataBuffer()
 	{
 	int ret = message->RequestBuffer(&data_buf_length, &data_buf_data);
-	if ( ! ret || data_buf_length == 0 || data_buf_data == 0 )
+	if ( ! ret || data_buf_length == 0 || data_buf_data == nullptr )
 		{
 		// reporter->InternalError("cannot get data buffer from MIME_Message", "");
-		return 0;
+		return false;
 		}
 
 	data_buf_offset = 0;
-	return 1;
+	return true;
 	}
 
 void MIME_Entity::DataOctet(char ch)
@@ -1211,7 +1214,7 @@ void MIME_Entity::DataOctets(int len, const char* data)
 		if ( data_buf_offset < 0 && ! GetDataBuffer() )
 			return;
 
-		int n = min(data_buf_length - data_buf_offset, len);
+		int n = std::min(data_buf_length - data_buf_offset, len);
 		memcpy(data_buf_data + data_buf_offset, data, n);
 		data += n;
 		data_buf_offset += n;
@@ -1246,18 +1249,18 @@ void MIME_Entity::SubmitAllHeaders()
 
 void MIME_Entity::BeginChildEntity()
 	{
-	ASSERT(current_child_entity == 0);
+	ASSERT(current_child_entity == nullptr);
 	current_child_entity = NewChildEntity();
 	message->BeginEntity(current_child_entity);
 	}
 
 void MIME_Entity::EndChildEntity()
 	{
-	ASSERT(current_child_entity != 0);
+	ASSERT(current_child_entity != nullptr);
 
 	current_child_entity->EndOfData();
 	delete current_child_entity;
-	current_child_entity = 0;
+	current_child_entity = nullptr;
 	}
 
 void MIME_Entity::IllegalFormat(const char* explanation)
@@ -1273,10 +1276,8 @@ void MIME_Entity::IllegalEncoding(const char* explanation)
 void MIME_Entity::DebugPrintHeaders()
 	{
 #ifdef DEBUG_BRO
-	for ( unsigned int i = 0; i < headers.size(); ++i )
+	for ( MIME_Header* h : headers )
 		{
-		MIME_Header* h = headers[i];
-
 		DEBUG_fputs(h->get_name(), stderr);
 		DEBUG_MSG(":\"");
 		DEBUG_fputs(h->get_value(), stderr);
@@ -1285,40 +1286,40 @@ void MIME_Entity::DebugPrintHeaders()
 #endif
 	}
 
-RecordVal* MIME_Message::BuildHeaderVal(MIME_Header* h)
+RecordValPtr MIME_Message::ToHeaderVal(MIME_Header* h)
 	{
-	RecordVal* header_record = new RecordVal(mime_header_rec);
-	header_record->Assign(0, new_string_val(h->get_name())->ToUpper());
-	header_record->Assign(1, new_string_val(h->get_value()));
+	static auto mime_header_rec = id::find_type<RecordType>("mime_header_rec");
+	auto header_record = make_intrusive<RecordVal>(mime_header_rec);
+	header_record->Assign(0, to_string_val(h->get_name()));
+	auto upper_hn = to_string_val(h->get_name());
+	upper_hn->ToUpper();
+	header_record->Assign(1, std::move(upper_hn));
+	header_record->Assign(2, to_string_val(h->get_value()));
 	return header_record;
 	}
 
-TableVal* MIME_Message::BuildHeaderTable(MIME_HeaderList& hlist)
+TableValPtr MIME_Message::ToHeaderTable(MIME_HeaderList& hlist)
 	{
-	TableVal* t = new TableVal(mime_header_list);
+	static auto mime_header_list = id::find_type<TableType>("mime_header_list");
+	auto t = make_intrusive<TableVal>(mime_header_list);
 
-	for ( unsigned int i = 0; i < hlist.size(); ++i )
+	for ( size_t i = 0; i < hlist.size(); ++i )
 		{
-		Val* index = val_mgr->GetCount(i+1);	// index starting from 1
-
+		auto index = val_mgr->Count(i + 1); // index starting from 1
 		MIME_Header* h = hlist[i];
-		RecordVal* header_record = BuildHeaderVal(h);
-
-		t->Assign(index, header_record);
-
-		Unref(index);
+		t->Assign(std::move(index), ToHeaderVal(h));
 		}
 
 	return t;
 	}
 
 MIME_Mail::MIME_Mail(analyzer::Analyzer* mail_analyzer, bool orig, int buf_size)
-: MIME_Message(mail_analyzer), md5_hash()
+	: MIME_Message(mail_analyzer), md5_hash()
 	{
 	analyzer = mail_analyzer;
 
-	min_overlap_length = mime_segment_overlap_length;
-	max_chunk_length = mime_segment_length;
+	min_overlap_length = zeek::detail::mime_segment_overlap_length;
+	max_chunk_length = zeek::detail::mime_segment_length;
 	is_orig = orig;
 
 	int length = buf_size;
@@ -1333,19 +1334,19 @@ MIME_Mail::MIME_Mail(analyzer::Analyzer* mail_analyzer, bool orig, int buf_size)
 		length = max_chunk_length;
 
 	buffer_start = data_start = 0;
-	data_buffer = new BroString(1, new u_char[length+1], length);
+	data_buffer = new String(true, new u_char[length + 1], length);
 
 	if ( mime_content_hash )
 		{
 		compute_content_hash = 1;
-		md5_hash = hash_init(Hash_MD5);
+		md5_hash = zeek::detail::hash_init(detail::Hash_MD5);
 		}
 	else
 		compute_content_hash = 0;
 
 	content_hash_length = 0;
 
-	top_level = new MIME_Entity(this, 0);	// to be changed to MIME_Mail
+	top_level = new MIME_Entity(this, nullptr); // to be changed to MIME_Mail
 	BeginEntity(top_level);
 	}
 
@@ -1358,14 +1359,12 @@ void MIME_Mail::Done()
 	if ( compute_content_hash && mime_content_hash )
 		{
 		u_char* digest = new u_char[16];
-		hash_final(md5_hash, digest);
+		zeek::detail::hash_final(md5_hash, digest);
 		md5_hash = nullptr;
 
-		analyzer->ConnectionEventFast(mime_content_hash, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(content_hash_length),
-			new StringVal(new BroString(1, digest, 16)),
-		});
+		analyzer->EnqueueConnEvent(mime_content_hash, analyzer->ConnVal(),
+		                           val_mgr->Count(content_hash_length),
+		                           make_intrusive<StringVal>(new String(true, digest, 16)));
 		}
 
 	MIME_Message::Done();
@@ -1389,7 +1388,7 @@ void MIME_Mail::BeginEntity(MIME_Entity* /* entity */)
 	cur_entity_id.clear();
 
 	if ( mime_begin_entity )
-		analyzer->ConnectionEventFast(mime_begin_entity, {analyzer->BuildConnVal()});
+		analyzer->EnqueueConnEvent(mime_begin_entity, analyzer->ConnVal());
 
 	buffer_start = data_start = 0;
 	ASSERT(entity_content.size() == 0);
@@ -1399,13 +1398,10 @@ void MIME_Mail::EndEntity(MIME_Entity* /* entity */)
 	{
 	if ( mime_entity_data )
 		{
-		BroString* s = concatenate(entity_content);
+		String* s = concatenate(entity_content);
 
-		analyzer->ConnectionEventFast(mime_entity_data, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(s->Len()),
-			new StringVal(s),
-		});
+		analyzer->EnqueueConnEvent(mime_entity_data, analyzer->ConnVal(), val_mgr->Count(s->Len()),
+		                           make_intrusive<StringVal>(s));
 
 		if ( ! mime_all_data )
 			delete_strings(entity_content);
@@ -1414,7 +1410,7 @@ void MIME_Mail::EndEntity(MIME_Entity* /* entity */)
 		}
 
 	if ( mime_end_entity )
-		analyzer->ConnectionEventFast(mime_end_entity, {analyzer->BuildConnVal()});
+		analyzer->EnqueueConnEvent(mime_end_entity, analyzer->ConnVal());
 
 	file_mgr->EndOfFile(analyzer->GetAnalyzerTag(), analyzer->Conn());
 	cur_entity_id.clear();
@@ -1423,43 +1419,32 @@ void MIME_Mail::EndEntity(MIME_Entity* /* entity */)
 void MIME_Mail::SubmitHeader(MIME_Header* h)
 	{
 	if ( mime_one_header )
-		{
-		analyzer->ConnectionEventFast(mime_one_header, {
-			analyzer->BuildConnVal(),
-			BuildHeaderVal(h),
-		});
-		}
+		analyzer->EnqueueConnEvent(mime_one_header, analyzer->ConnVal(), ToHeaderVal(h));
 	}
 
 void MIME_Mail::SubmitAllHeaders(MIME_HeaderList& hlist)
 	{
 	if ( mime_all_headers )
-		{
-		analyzer->ConnectionEventFast(mime_all_headers, {
-			analyzer->BuildConnVal(),
-			BuildHeaderTable(hlist),
-		});
-		}
+		analyzer->EnqueueConnEvent(mime_all_headers, analyzer->ConnVal(), ToHeaderTable(hlist));
 	}
 
 void MIME_Mail::SubmitData(int len, const char* buf)
 	{
-	if ( buf != (char*) data_buffer->Bytes() + buffer_start )
+	if ( buf != (char*)data_buffer->Bytes() + buffer_start )
 		{
-		reporter->AnalyzerError(GetAnalyzer(),
-		                                "MIME buffer misalignment");
+		reporter->AnalyzerError(GetAnalyzer(), "MIME buffer misalignment");
 		return;
 		}
 
 	if ( compute_content_hash )
 		{
 		content_hash_length += len;
-		hash_update(md5_hash, (const u_char*) buf, len);
+		zeek::detail::hash_update(md5_hash, (const u_char*)buf, len);
 		}
 
 	if ( mime_entity_data || mime_all_data )
 		{
-		BroString* s = new BroString((const u_char*) buf, len, 0);
+		String* s = new String((const u_char*)buf, len, false);
 
 		if ( mime_entity_data )
 			entity_content.push_back(s);
@@ -1469,25 +1454,22 @@ void MIME_Mail::SubmitData(int len, const char* buf)
 
 	if ( mime_segment_data )
 		{
-		const char* data = (char*) data_buffer->Bytes() + data_start;
+		const char* data = (char*)data_buffer->Bytes() + data_start;
 		int data_len = (buf + len) - data;
 
-		analyzer->ConnectionEventFast(mime_segment_data, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(data_len),
-			new StringVal(data_len, data),
-		});
+		analyzer->EnqueueConnEvent(mime_segment_data, analyzer->ConnVal(), val_mgr->Count(data_len),
+		                           make_intrusive<StringVal>(data_len, data));
 		}
 
 	cur_entity_id = file_mgr->DataIn(reinterpret_cast<const u_char*>(buf), len,
-	                 analyzer->GetAnalyzerTag(), analyzer->Conn(), is_orig,
-	                 cur_entity_id);
+	                                 analyzer->GetAnalyzerTag(), analyzer->Conn(), is_orig,
+	                                 cur_entity_id);
 
 	cur_entity_len += len;
 	buffer_start = (buf + len) - (char*)data_buffer->Bytes();
 	}
 
-int MIME_Mail::RequestBuffer(int* plen, char** pbuf)
+bool MIME_Mail::RequestBuffer(int* plen, char** pbuf)
 	{
 	data_start = buffer_start - min_overlap_length;
 	if ( data_start < 0 )
@@ -1500,30 +1482,26 @@ int MIME_Mail::RequestBuffer(int* plen, char** pbuf)
 		// Copy every thing in [data_start, buffer_start) to
 		// [0, overlap).
 		if ( buffer_start > data_start )
-			memcpy(data_buffer->Bytes(),
-				data_buffer->Bytes() + data_start, overlap);
+			memcpy(data_buffer->Bytes(), data_buffer->Bytes() + data_start, overlap);
 		data_start = 0;
 		buffer_start = overlap;
 		}
 
 	*plen = max_chunk_length - overlap;
-	*pbuf = (char*) data_buffer->Bytes() + buffer_start;
+	*pbuf = (char*)data_buffer->Bytes() + buffer_start;
 
-	return 1;
+	return true;
 	}
 
 void MIME_Mail::SubmitAllData()
 	{
 	if ( mime_all_data )
 		{
-		BroString* s = concatenate(all_content);
+		String* s = concatenate(all_content);
 		delete_strings(all_content);
 
-		analyzer->ConnectionEventFast(mime_all_data, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(s->Len()),
-			new StringVal(s),
-		});
+		analyzer->EnqueueConnEvent(mime_all_data, analyzer->ConnVal(), val_mgr->Count(s->Len()),
+		                           make_intrusive<StringVal>(s));
 		}
 	}
 
@@ -1531,7 +1509,8 @@ void MIME_Mail::SubmitEvent(int event_type, const char* detail)
 	{
 	const char* category = "";
 
-	switch ( event_type ) {
+	switch ( event_type )
+		{
 		case MIME_EVENT_ILLEGAL_FORMAT:
 			category = "illegal format";
 			break;
@@ -1541,17 +1520,14 @@ void MIME_Mail::SubmitEvent(int event_type, const char* detail)
 			break;
 
 		default:
-			reporter->AnalyzerError(GetAnalyzer(),
-			                                "unrecognized MIME_Mail event");
+			reporter->AnalyzerError(GetAnalyzer(), "unrecognized MIME_Mail event");
 			return;
-	}
+		}
 
 	if ( mime_event )
-		{
-		analyzer->ConnectionEventFast(mime_event, {
-			analyzer->BuildConnVal(),
-			new StringVal(category),
-			new StringVal(detail),
-		});
-		}
+		analyzer->EnqueueConnEvent(mime_event, analyzer->ConnVal(),
+		                           make_intrusive<StringVal>(category),
+		                           make_intrusive<StringVal>(detail));
 	}
+
+	} // namespace zeek::analyzer::mime
