@@ -1,33 +1,37 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
+#include "zeek/analyzer/protocol/rpc/RPC.h"
+
+#include "zeek/zeek-config.h"
+
 #include <stdlib.h>
+#include <string>
 
-#include <algorithm>
+#include "zeek/NetVar.h"
+#include "zeek/Reporter.h"
+#include "zeek/RunState.h"
+#include "zeek/analyzer/protocol/rpc/XDR.h"
+#include "zeek/analyzer/protocol/rpc/events.bif.h"
+#include "zeek/session/Manager.h"
 
-#include "zeek-config.h"
-
-#include "NetVar.h"
-#include "XDR.h"
-#include "RPC.h"
-#include "Sessions.h"
-
-#include "events.bif.h"
-
-using namespace analyzer::rpc;
-
-namespace { // local namespace
-	const bool DEBUG_rpc_resync = false;
-}
+namespace
+	{ // local namespace
+const bool DEBUG_rpc_resync = false;
+	}
 
 // TODO: Should we add start_time and last_time to the rpc_* events??
 
 // TODO: make this configurable
 #define MAX_RPC_LEN 65536
 
+namespace zeek::analyzer::rpc
+	{
+namespace detail
+	{
 
-RPC_CallInfo::RPC_CallInfo(uint32 arg_xid, const u_char*& buf, int& n, double arg_start_time, double arg_last_time, int arg_rpc_len)
+RPC_CallInfo::RPC_CallInfo(uint32_t arg_xid, const u_char*& buf, int& n, double arg_start_time,
+                           double arg_last_time, int arg_rpc_len)
 	{
-	v = nullptr;
 	xid = arg_xid;
 	stamp = 0;
 	uid = 0;
@@ -38,7 +42,7 @@ RPC_CallInfo::RPC_CallInfo(uint32 arg_xid, const u_char*& buf, int& n, double ar
 	rpc_len = arg_rpc_len;
 	call_n = n;
 	call_buf = new u_char[call_n];
-	memcpy((void*) call_buf, (const void*) buf, call_n);
+	memcpy((void*)call_buf, (const void*)buf, call_n);
 
 	rpc_version = extract_XDR_uint32(buf, n);
 	prog = extract_XDR_uint32(buf, n);
@@ -65,7 +69,8 @@ RPC_CallInfo::RPC_CallInfo(uint32 arg_xid, const u_char*& buf, int& n, double ar
 		stamp = extract_XDR_uint32(cred_opaque, cred_opaque_n);
 		int machinename_n;
 		constexpr auto max_machinename_len = 255;
-		auto mnp = extract_XDR_opaque(cred_opaque, cred_opaque_n, machinename_n, max_machinename_len);
+		auto mnp = extract_XDR_opaque(cred_opaque, cred_opaque_n, machinename_n,
+		                              max_machinename_len);
 
 		if ( ! mnp )
 			{
@@ -85,7 +90,7 @@ RPC_CallInfo::RPC_CallInfo(uint32 arg_xid, const u_char*& buf, int& n, double ar
 			return;
 			}
 
-		for ( auto i = 0u; i < number_of_gids; ++i )
+		for ( size_t i = 0; i < number_of_gids; ++i )
 			auxgids.push_back(extract_XDR_uint32(cred_opaque, cred_opaque_n));
 		}
 
@@ -96,46 +101,42 @@ RPC_CallInfo::RPC_CallInfo(uint32 arg_xid, const u_char*& buf, int& n, double ar
 
 RPC_CallInfo::~RPC_CallInfo()
 	{
-	delete [] call_buf;
-	Unref(v);
+	delete[] call_buf;
 	}
 
-int RPC_CallInfo::CompareRexmit(const u_char* buf, int n) const
+bool RPC_CallInfo::CompareRexmit(const u_char* buf, int n) const
 	{
 	if ( n != call_n )
-		return 0;
+		return false;
 
-	return memcmp((const void*) call_buf, (const void*) buf, call_n) == 0;
-	}
-
-
-void rpc_callinfo_delete_func(void* v)
-	{
-	delete (RPC_CallInfo*) v;
+	return memcmp((const void*)call_buf, (const void*)buf, call_n) == 0;
 	}
 
 RPC_Interpreter::RPC_Interpreter(analyzer::Analyzer* arg_analyzer)
 	{
 	analyzer = arg_analyzer;
-	calls.SetDeleteFunc(rpc_callinfo_delete_func);
 	}
 
 RPC_Interpreter::~RPC_Interpreter()
 	{
+	for ( const auto& call : calls )
+		delete call.second;
 	}
 
-int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
-				int is_orig, double start_time, double last_time)
+int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen, bool is_orig,
+                                double start_time, double last_time)
 	{
-	uint32 xid = extract_XDR_uint32(buf, n);
-	uint32 msg_type = extract_XDR_uint32(buf, n);
+	uint32_t xid = extract_XDR_uint32(buf, n);
+	uint32_t msg_type = extract_XDR_uint32(buf, n);
 	int rpc_len = n;
 
 	if ( ! buf )
 		return 0;
 
-	HashKey h(&xid, 1);
-	RPC_CallInfo* call = calls.Lookup(&h);
+	RPC_CallInfo* call = nullptr;
+	auto iter = calls.find(xid);
+	if ( iter != calls.end() )
+		call = iter->second;
 
 	if ( msg_type == RPC_CALL )
 		{
@@ -175,7 +176,7 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 				return 0;
 				}
 
-			calls.Insert(&h, call);
+			calls[xid] = call;
 			}
 
 		// We now have a valid RPC_CallInfo (either the previous one
@@ -197,7 +198,7 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 		if ( is_orig )
 			Weird("originator_RPC_reply");
 
-		uint32 reply_stat = extract_XDR_uint32(buf, n);
+		uint32_t reply_stat = extract_XDR_uint32(buf, n);
 		if ( ! buf )
 			return 0;
 
@@ -205,8 +206,8 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 
 		if ( reply_stat == RPC_MSG_ACCEPTED )
 			{
-			(void) skip_XDR_opaque_auth(buf, n);
-			uint32 accept_stat = extract_XDR_uint32(buf, n);
+			(void)skip_XDR_opaque_auth(buf, n);
+			uint32_t accept_stat = extract_XDR_uint32(buf, n);
 
 			// The first members of BifEnum::RPC_* correspond
 			// to accept_stat.
@@ -218,8 +219,8 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 
 			if ( accept_stat == RPC_PROG_MISMATCH )
 				{
-				(void) extract_XDR_uint32(buf, n);
-				(void) extract_XDR_uint32(buf, n);
+				(void)extract_XDR_uint32(buf, n);
+				(void)extract_XDR_uint32(buf, n);
 
 				if ( ! buf )
 					return 0;
@@ -228,7 +229,7 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 
 		else if ( reply_stat == RPC_MSG_DENIED )
 			{
-			uint32 reject_stat = extract_XDR_uint32(buf, n);
+			uint32_t reject_stat = extract_XDR_uint32(buf, n);
 			if ( ! buf )
 				return 0;
 
@@ -237,8 +238,8 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 				// Note that RPC_MISMATCH == 0 == RPC_SUCCESS.
 				status = BifEnum::RPC_VERS_MISMATCH;
 
-				(void) extract_XDR_uint32(buf, n);
-				(void) extract_XDR_uint32(buf, n);
+				(void)extract_XDR_uint32(buf, n);
+				(void)extract_XDR_uint32(buf, n);
 
 				if ( ! buf )
 					return 0;
@@ -248,7 +249,7 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 				{
 				status = BifEnum::RPC_AUTH_ERROR;
 
-				(void) extract_XDR_uint32(buf, n);
+				(void)extract_XDR_uint32(buf, n);
 				if ( ! buf )
 					return 0;
 				}
@@ -279,13 +280,15 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 
 			else
 				{
-				if ( ! RPC_BuildReply(call, (BifEnum::rpc_status)status, buf, n, start_time, last_time, rpc_len) )
+				if ( ! RPC_BuildReply(call, (BifEnum::rpc_status)status, buf, n, start_time,
+				                      last_time, rpc_len) )
 					Weird("bad_RPC");
 				}
 
 			Event_RPC_Dialogue(call, status, n);
 
-			delete calls.RemoveEntry(&h);
+			calls.erase(xid);
+			delete call;
 			}
 		else
 			{
@@ -319,11 +322,9 @@ int RPC_Interpreter::DeliverRPC(const u_char* buf, int n, int rpclen,
 
 void RPC_Interpreter::Timeout()
 	{
-	IterCookie* cookie = calls.InitForIteration();
-	RPC_CallInfo* c;
-
-	while ( (c = calls.NextEntry(cookie)) )
+	for ( const auto& entry : calls )
 		{
+		RPC_CallInfo* c = entry.second;
 		Event_RPC_Dialogue(c, BifEnum::RPC_TIMEOUT, 0);
 
 		if ( c->IsValidCall() )
@@ -331,7 +332,8 @@ void RPC_Interpreter::Timeout()
 			const u_char* buf = nullptr;
 			int n = 0;
 
-			if ( ! RPC_BuildReply(c, BifEnum::RPC_TIMEOUT, buf, n, network_time, network_time, 0) )
+			if ( ! RPC_BuildReply(c, BifEnum::RPC_TIMEOUT, buf, n, run_state::network_time,
+			                      run_state::network_time, 0) )
 				Weird("bad_RPC");
 			}
 		}
@@ -340,46 +342,27 @@ void RPC_Interpreter::Timeout()
 void RPC_Interpreter::Event_RPC_Dialogue(RPC_CallInfo* c, BifEnum::rpc_status status, int reply_len)
 	{
 	if ( rpc_dialogue )
-		{
-		analyzer->ConnectionEventFast(rpc_dialogue, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(c->Program()),
-			val_mgr->GetCount(c->Version()),
-			val_mgr->GetCount(c->Proc()),
-			BifType::Enum::rpc_status->GetVal(status),
-			new Val(c->StartTime(), TYPE_TIME),
-			val_mgr->GetCount(c->CallLen()),
-			val_mgr->GetCount(reply_len),
-		});
-		}
+		analyzer->EnqueueConnEvent(rpc_dialogue, analyzer->ConnVal(), val_mgr->Count(c->Program()),
+		                           val_mgr->Count(c->Version()), val_mgr->Count(c->Proc()),
+		                           BifType::Enum::rpc_status->GetEnumVal(status),
+		                           make_intrusive<TimeVal>(c->StartTime()),
+		                           val_mgr->Count(c->CallLen()), val_mgr->Count(reply_len));
 	}
 
 void RPC_Interpreter::Event_RPC_Call(RPC_CallInfo* c)
 	{
 	if ( rpc_call )
-		{
-		analyzer->ConnectionEventFast(rpc_call, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(c->XID()),
-			val_mgr->GetCount(c->Program()),
-			val_mgr->GetCount(c->Version()),
-			val_mgr->GetCount(c->Proc()),
-			val_mgr->GetCount(c->CallLen()),
-		});
-		}
+		analyzer->EnqueueConnEvent(rpc_call, analyzer->ConnVal(), val_mgr->Count(c->XID()),
+		                           val_mgr->Count(c->Program()), val_mgr->Count(c->Version()),
+		                           val_mgr->Count(c->Proc()), val_mgr->Count(c->CallLen()));
 	}
 
 void RPC_Interpreter::Event_RPC_Reply(uint32_t xid, BifEnum::rpc_status status, int reply_len)
 	{
 	if ( rpc_reply )
-		{
-		analyzer->ConnectionEventFast(rpc_reply, {
-			analyzer->BuildConnVal(),
-			val_mgr->GetCount(xid),
-			BifType::Enum::rpc_status->GetVal(status),
-			val_mgr->GetCount(reply_len),
-		});
-		}
+		analyzer->EnqueueConnEvent(rpc_reply, analyzer->ConnVal(), val_mgr->Count(xid),
+		                           BifType::Enum::rpc_status->GetEnumVal(status),
+		                           val_mgr->Count(reply_len));
 	}
 
 void RPC_Interpreter::Weird(const char* msg, const char* addl)
@@ -387,31 +370,31 @@ void RPC_Interpreter::Weird(const char* msg, const char* addl)
 	analyzer->Weird(msg, addl);
 	}
 
-
-void RPC_Reasm_Buffer::Init(int64_t arg_maxsize, int64_t arg_expected) {
+void RPC_Reasm_Buffer::Init(int64_t arg_maxsize, int64_t arg_expected)
+	{
 	if ( buf )
-		delete [] buf;
+		delete[] buf;
 	expected = arg_expected;
 	maxsize = arg_maxsize;
 	fill = processed = 0;
 	buf = new u_char[maxsize];
-};
+	};
 
 bool RPC_Reasm_Buffer::ConsumeChunk(const u_char*& data, int& len)
 	{
 	// How many bytes do we want to process with this call?  Either the
 	// all of the bytes available or the number of bytes that we are
 	// still missing.
-	int64_t to_process = min(int64_t(len), (expected-processed));
+	int64_t to_process = std::min(int64_t(len), (expected - processed));
 
 	if ( fill < maxsize )
 		{
 		// We haven't yet filled the buffer. How many bytes to copy
 		// into the buff. Either all of the bytes we want to process
 		// or the number of bytes until we reach maxsize.
-		int64_t to_copy = min( to_process, (maxsize-fill) );
+		int64_t to_copy = std::min(to_process, (maxsize - fill));
 		if ( to_copy )
-			memcpy(buf+fill, data, to_copy);
+			memcpy(buf + fill, data, to_copy);
 
 		fill += to_copy;
 		}
@@ -422,9 +405,10 @@ bool RPC_Reasm_Buffer::ConsumeChunk(const u_char*& data, int& len)
 	return (expected == processed);
 	}
 
-Contents_RPC::Contents_RPC(Connection* conn, bool orig,
-				RPC_Interpreter* arg_interp)
-	: tcp::TCP_SupportAnalyzer("CONTENTS_RPC", conn, orig)
+	} // namespace detail
+
+Contents_RPC::Contents_RPC(Connection* conn, bool orig, detail::RPC_Interpreter* arg_interp)
+	: analyzer::tcp::TCP_SupportAnalyzer("CONTENTS_RPC", conn, orig)
 	{
 	interp = arg_interp;
 	state = WAIT_FOR_MESSAGE;
@@ -436,45 +420,42 @@ Contents_RPC::Contents_RPC(Connection* conn, bool orig,
 
 void Contents_RPC::Init()
 	{
-	tcp::TCP_SupportAnalyzer::Init();
+	analyzer::tcp::TCP_SupportAnalyzer::Init();
 	}
 
-Contents_RPC::~Contents_RPC()
-	{
-	}
+Contents_RPC::~Contents_RPC() { }
 
-void Contents_RPC::Undelivered(uint64 seq, int len, bool orig)
+void Contents_RPC::Undelivered(uint64_t seq, int len, bool orig)
 	{
-	tcp::TCP_SupportAnalyzer::Undelivered(seq, len, orig);
+	analyzer::tcp::TCP_SupportAnalyzer::Undelivered(seq, len, orig);
 	NeedResync();
 	}
 
 bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 	{
-	uint32 frame_len;
+	uint32_t frame_len;
 	bool last_frag;
-	uint32 xid;
-	uint32 frame_type;
+	uint32_t xid;
+	uint32_t frame_type;
 
 	bool discard_this_chunk = false;
 
 	if ( resync_state == RESYNC_INIT )
-		{ 
+		{
 		// First time CheckResync is called. If the TCP endpoint
-		// is fully established we are in sync (since it's the first chunk 
-		// of data after the SYN if its not established we need to 
+		// is fully established we are in sync (since it's the first chunk
+		// of data after the SYN if its not established we need to
 		// resync.
-		tcp::TCP_Analyzer* tcp =
-			static_cast<tcp::TCP_ApplicationAnalyzer*>(Parent())->TCP();
-		assert(tcp);
+		resync_state = INSYNC;
 
-		if ( (IsOrig() ? tcp->OrigState() : tcp->RespState()) !=
-							tcp::TCP_ENDPOINT_ESTABLISHED )
+		if ( auto* tcp = static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(Parent())->TCP() )
 			{
-			NeedResync();
+			if ( (IsOrig() ? tcp->OrigState() : tcp->RespState()) !=
+			     analyzer::tcp::TCP_ENDPOINT_ESTABLISHED )
+				{
+				NeedResync();
+				}
 			}
-		else
-			resync_state = INSYNC;
 		}
 
 	if ( resync_state == INSYNC )
@@ -497,7 +478,7 @@ bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 
 	// Assuming RPC frames align with packet boundaries ...
 
-	while (len > 0)
+	while ( len > 0 )
 		{
 		if ( resync_toskip )
 			{
@@ -523,8 +504,7 @@ bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 		if ( resync_toskip != 0 )
 			{
 			// Should never happen.
-			reporter->AnalyzerError(this,
-			  "RPC resync: skipping over data failed");
+			reporter->AnalyzerError(this, "RPC resync: skipping over data failed");
 			return false;
 			}
 
@@ -532,7 +512,6 @@ bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 		// frame. If the resync processs is successful, we should be
 		// at the beginning of a frame.
 
-
 		if ( len < 12 )
 			{
 			// Ignore small chunks.
@@ -541,17 +520,16 @@ bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 				// One-byte fragments are likely caused by
 				// TCP keep-alive retransmissions.
 				DEBUG_MSG("%.6f RPC resync: "
-						  "discard small pieces: %d\n",
-							  network_time, len);
-				Conn()->Weird("RPC_resync", fmt("discard %d bytes\n", len));
+				          "discard small pieces: %d\n",
+				          run_state::network_time, len);
+				Conn()->Weird("RPC_resync", util::fmt("discard %d bytes\n", len));
 				}
 
 			NeedResync();
 			return false;
 			}
 
-
-		const u_char *xdata = data;
+		const u_char* xdata = data;
 		int xlen = len;
 		frame_len = extract_XDR_uint32(xdata, xlen);
 		last_frag = (frame_len & 0x80000000) != 0;
@@ -561,9 +539,7 @@ bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 
 		// Check if the direction makes sense and the length of the
 		// frame to expect.
-		if ( (IsOrig() && frame_type != 0) ||
-			 (! IsOrig() && frame_type != 1) ||
-			 frame_len < 16 )
+		if ( (IsOrig() && frame_type != 0) || (! IsOrig() && frame_type != 1) || frame_len < 16 )
 
 			discard_this_chunk = true;
 
@@ -578,166 +554,167 @@ bool Contents_RPC::CheckResync(int& len, const u_char*& data, bool orig)
 			if ( DEBUG_rpc_resync )
 				DEBUG_MSG("RPC resync: Need to resync. dicarding %d bytes.\n", len);
 
-			NeedResync();  // let's try the resync again from the beginning
+			NeedResync(); // let's try the resync again from the beginning
 			return false;
 			}
 
 		// Looks like we are at the start of a frame and have successfully
 		// extracted the frame length (marker).
 
-		switch (resync_state) {
-		case NEED_RESYNC:
-		case RESYNC_WAIT_FOR_MSG_START:
-			// Initial phase of resyncing. Skip frames until we get a frame
-			// with the last_fragment bit set.
-			resync_toskip = frame_len + 4;
+		switch ( resync_state )
+			{
+			case NEED_RESYNC:
+			case RESYNC_WAIT_FOR_MSG_START:
+				// Initial phase of resyncing. Skip frames until we get a frame
+				// with the last_fragment bit set.
+				resync_toskip = frame_len + 4;
 
-			if ( last_frag )
-				resync_state = RESYNC_WAIT_FOR_FULL_MSG;
-			else
-				resync_state = RESYNC_WAIT_FOR_MSG_START;
-			break;
+				if ( last_frag )
+					resync_state = RESYNC_WAIT_FOR_FULL_MSG;
+				else
+					resync_state = RESYNC_WAIT_FOR_MSG_START;
+				break;
 
-		case RESYNC_WAIT_FOR_FULL_MSG:
-			// If the resync was successful so far, we should now be the start
-			// of a new RPC message. Try to skip over it.
-			resync_toskip = frame_len + 4;
+			case RESYNC_WAIT_FOR_FULL_MSG:
+				// If the resync was successful so far, we should now be the start
+				// of a new RPC message. Try to skip over it.
+				resync_toskip = frame_len + 4;
 
-			if ( last_frag )
-				resync_state = RESYNC_HAD_FULL_MSG;
-			break;
+				if ( last_frag )
+					resync_state = RESYNC_HAD_FULL_MSG;
+				break;
 
-		case RESYNC_HAD_FULL_MSG:
-			// We have now successfully skipped over a full RPC message.
-			// If we got that far, we are in sync.
-			resync_state = INSYNC;
+			case RESYNC_HAD_FULL_MSG:
+				// We have now successfully skipped over a full RPC message.
+				// If we got that far, we are in sync.
+				resync_state = INSYNC;
 
-			if ( DEBUG_rpc_resync )
-				DEBUG_MSG("RPC resync: success.\n");
-			return true;
+				if ( DEBUG_rpc_resync )
+					DEBUG_MSG("RPC resync: success.\n");
+				return true;
 
-		default:
-			// Should never happen.
-			NeedResync();
-			return false;
-		} // end switch
+			default:
+				// Should never happen.
+				NeedResync();
+				return false;
+			} // end switch
 		} // end while (len>0)
 
 	return false;
 	}
 
-
-
-
 void Contents_RPC::DeliverStream(int len, const u_char* data, bool orig)
 	{
-	tcp::TCP_SupportAnalyzer::DeliverStream(len, data, orig);
-	uint32 marker;
+	analyzer::tcp::TCP_SupportAnalyzer::DeliverStream(len, data, orig);
+	uint32_t marker;
 	bool last_frag;
 
 	if ( ! CheckResync(len, data, orig) )
-		return;	// Not in sync yet. Still resyncing.
+		return; // Not in sync yet. Still resyncing.
 
 	// Should be in sync now.
 
-	while (len > 0)
+	while ( len > 0 )
 		{
-		last_time = network_time;
+		last_time = run_state::network_time;
 
-		switch (state) {
-		case WAIT_FOR_MESSAGE:
-			// A new RPC message is starting. Initialize state.
-
-			// We expect and want 4 bytes of the frame markers.
-			marker_buf.Init(4,4);
-
-			// We want at most 64KB of message data and we don't
-			// know yet how much we expect, so we set expected to
-			// 0.
-			msg_buf.Init(MAX_RPC_LEN, 0);
-			last_frag = 0;
-			state = WAIT_FOR_MARKER;
-			start_time = network_time;
-			// no break. fall through
-
-		case WAIT_FOR_MARKER:
+		switch ( state )
 			{
-			bool got_marker = marker_buf.ConsumeChunk(data,len);
+			case WAIT_FOR_MESSAGE:
+				// A new RPC message is starting. Initialize state.
 
-			if ( got_marker )
+				// We expect and want 4 bytes of the frame markers.
+				marker_buf.Init(4, 4);
+
+				// We want at most 64KB of message data and we don't
+				// know yet how much we expect, so we set expected to
+				// 0.
+				msg_buf.Init(MAX_RPC_LEN, 0);
+				last_frag = false;
+				state = WAIT_FOR_MARKER;
+				start_time = run_state::network_time;
+				// no break. fall through
+
+			case WAIT_FOR_MARKER:
 				{
-				const u_char *dummy_p = marker_buf.GetBuf();
-				int dummy_len = (int) marker_buf.GetFill();
+				bool got_marker = marker_buf.ConsumeChunk(data, len);
 
-				// have full marker
-				marker = extract_XDR_uint32(dummy_p, dummy_len);
-				marker_buf.Init(4,4);
-
-				if ( ! dummy_p )
+				if ( got_marker )
 					{
-					reporter->AnalyzerError(this,
-					  "inconsistent RPC record marker extraction");
-					return;
+					const u_char* dummy_p = marker_buf.GetBuf();
+					int dummy_len = (int)marker_buf.GetFill();
+
+					// have full marker
+					marker = extract_XDR_uint32(dummy_p, dummy_len);
+					marker_buf.Init(4, 4);
+
+					if ( ! dummy_p )
+						{
+						reporter->AnalyzerError(this, "inconsistent RPC record marker extraction");
+						return;
+						}
+
+					last_frag = (marker & 0x80000000) != 0;
+					marker &= 0x7fffffff;
+					// printf("%.6f %d marker= %u <> last_frag= %d <> expected=%llu <>
+					// processed= %llu <> len = %d\n", 		run_state::network_time, IsOrig(),
+					// marker,
+					// last_frag, msg_buf.GetExpected(), msg_buf.GetProcessed(), len);
+
+					if ( ! msg_buf.AddToExpected(marker) )
+						Conn()->Weird("RPC_message_too_long",
+						              util::fmt("%" PRId64, msg_buf.GetExpected()));
+
+					if ( last_frag )
+						state = WAIT_FOR_LAST_DATA;
+					else
+						state = WAIT_FOR_DATA;
 					}
-
-				last_frag = (marker & 0x80000000) != 0;
-				marker &= 0x7fffffff;
-					//printf("%.6f %d marker= %u <> last_frag= %d <> expected=%llu <> processed= %llu <> len = %d\n",
-					//		network_time, IsOrig(), marker, last_frag, msg_buf.GetExpected(), msg_buf.GetProcessed(), len);
-
-				if ( ! msg_buf.AddToExpected(marker) )
-					Conn()->Weird("RPC_message_too_long", fmt("%" PRId64, msg_buf.GetExpected()));
-
-				if ( last_frag )
-					state = WAIT_FOR_LAST_DATA;
-				else
-					state = WAIT_FOR_DATA;
 				}
-			}
-			// Else remain in state. Haven't got the full 4 bytes
-			// for the marker yet.
-			break;
+				// Else remain in state. Haven't got the full 4 bytes
+				// for the marker yet.
+				break;
 
-		case WAIT_FOR_DATA:
-		case WAIT_FOR_LAST_DATA:
-			{
-			bool got_all_data = msg_buf.ConsumeChunk(data, len);
-
-			if ( got_all_data )
+			case WAIT_FOR_DATA:
+			case WAIT_FOR_LAST_DATA:
 				{
-				// Got all the data we expected. Now let's
-				// see whether there is another fragment
-				// coming or whether we just finished the
-				// last fragment.
-				if ( state == WAIT_FOR_LAST_DATA )
+				bool got_all_data = msg_buf.ConsumeChunk(data, len);
+
+				if ( got_all_data )
 					{
-					const u_char *dummy_p = msg_buf.GetBuf();
-					int dummy_len = (int) msg_buf.GetFill();
+					// Got all the data we expected. Now let's
+					// see whether there is another fragment
+					// coming or whether we just finished the
+					// last fragment.
+					if ( state == WAIT_FOR_LAST_DATA )
+						{
+						const u_char* dummy_p = msg_buf.GetBuf();
+						int dummy_len = (int)msg_buf.GetFill();
 
-					if ( ! interp->DeliverRPC(dummy_p, dummy_len, (int)msg_buf.GetExpected(), IsOrig(), start_time, last_time) )
-						Conn()->Weird("partial_RPC");
+						if ( ! interp->DeliverRPC(dummy_p, dummy_len, (int)msg_buf.GetExpected(),
+						                          IsOrig(), start_time, last_time) )
+							Conn()->Weird("partial_RPC");
 
-					state = WAIT_FOR_MESSAGE;
+						state = WAIT_FOR_MESSAGE;
+						}
+					else
+						state = WAIT_FOR_MARKER;
 					}
-				else
-					state = WAIT_FOR_MARKER;
+				// Else remain in state. Haven't read all the data
+				// yet.
 				}
-			// Else remain in state. Haven't read all the data
-			// yet.
-			}
-			break;
-		} // end switch
+				break;
+			} // end switch
 		} // end while
 	}
 
-RPC_Analyzer::RPC_Analyzer(const char* name, Connection* conn,
-				RPC_Interpreter* arg_interp)
-	: tcp::TCP_ApplicationAnalyzer(name, conn),
-	  interp(arg_interp), orig_rpc(), resp_rpc()
+RPC_Analyzer::RPC_Analyzer(const char* name, Connection* conn, detail::RPC_Interpreter* arg_interp)
+	: analyzer::tcp::TCP_ApplicationAnalyzer(name, conn), interp(arg_interp), orig_rpc(), resp_rpc()
 	{
 	if ( Conn()->ConnTransport() == TRANSPORT_UDP )
 		ADD_ANALYZER_TIMER(&RPC_Analyzer::ExpireTimer,
-			network_time + rpc_timeout, 1, TIMER_RPC_EXPIRE);
+		                   run_state::network_time + zeek::detail::rpc_timeout, true,
+		                   zeek::detail::TIMER_RPC_EXPIRE);
 	}
 
 RPC_Analyzer::~RPC_Analyzer()
@@ -745,27 +722,29 @@ RPC_Analyzer::~RPC_Analyzer()
 	delete interp;
 	}
 
-void RPC_Analyzer::DeliverPacket(int len, const u_char* data, bool orig,
-					uint64 seq, const IP_Hdr* ip, int caplen)
+void RPC_Analyzer::DeliverPacket(int len, const u_char* data, bool orig, uint64_t seq,
+                                 const IP_Hdr* ip, int caplen)
 	{
-	tcp::TCP_ApplicationAnalyzer::DeliverPacket(len, data, orig, seq, ip, caplen);
-	len = min(len, caplen);
+	analyzer::tcp::TCP_ApplicationAnalyzer::DeliverPacket(len, data, orig, seq, ip, caplen);
+	len = std::min(len, caplen);
 
 	if ( orig )
 		{
-		if ( ! interp->DeliverRPC(data, len, len, 1, network_time, network_time) )
+		if ( ! interp->DeliverRPC(data, len, len, true, run_state::network_time,
+		                          run_state::network_time) )
 			Weird("bad_RPC");
 		}
 	else
 		{
-		if ( ! interp->DeliverRPC(data, len, len, 0, network_time, network_time) )
+		if ( ! interp->DeliverRPC(data, len, len, false, run_state::network_time,
+		                          run_state::network_time) )
 			Weird("bad_RPC");
 		}
 	}
 
 void RPC_Analyzer::Done()
 	{
-	tcp::TCP_ApplicationAnalyzer::Done();
+	analyzer::tcp::TCP_ApplicationAnalyzer::Done();
 
 	interp->Timeout();
 	}
@@ -773,5 +752,7 @@ void RPC_Analyzer::Done()
 void RPC_Analyzer::ExpireTimer(double /* t */)
 	{
 	Event(connection_timeout);
-	sessions->Remove(Conn());
+	session_mgr->Remove(Conn());
 	}
+
+	} // namespace zeek::analyzer::rpc
